from typing import Optional, List
from collections import defaultdict
from random import choice
from sc2.player import Race
from sharpy.knowledges import KnowledgeBot
from sharpy.managers import ManagerBase
from sharpy.plans import BuildOrder
from sharpy.plans.tactics.zerg import CounterTerranTie
from paul_plans.cheese import LingRush, NydusRush, RoachRavagerNydus
from paul_plans.managers import OlScoutManager
from paul_plans import MacroBuild
from paul_plans.opening_template import GenericMacroOpening


class PaulBot(KnowledgeBot):
    build_name: str

    def __init__(self):
        self.my_race = Race.Zerg
        super().__init__("Paul")

    async def create_plan(self) -> "BuildOrder":
        allowed_builds = ["LingRush", "NydusRush", "RoachRavagerNydus", "Macro"]
        if self.knowledge.data_manager.last_result:
            if self.knowledge.data_manager.last_result == 1 and self.knowledge.data_manager.last_result.build_used:
                self.build_name = self.knowledge.data_manager.last_result.build_used
            else:
                allowed_dict = defaultdict(int)
                for result in self.knowledge.data_manager.data.results:
                    build = result.build_used
                    if build in allowed_builds:
                        allowed_dict[build] += result.result
                best_result = max(allowed_dict.values())
                possible = []
                for b in allowed_dict:
                    if allowed_dict[b] == best_result:
                        possible.append(b)
                if possible:
                    self.build_name = choice(possible)
                else:
                    allowed_builds.remove(self.knowledge.data_manager.last_result.build_used)
                    self.build_name = choice(allowed_builds)
        else:
            self.build_name = choice(allowed_builds)
        self.knowledge.data_manager.set_build(self.build_name)

        if self.build_name == "LingRush":
            order = LingRush()
        elif self.build_name == "NydusRush":
            order = CounterTerranTie([NydusRush()])
        elif self.build_name == "RoachRavagerNydus":
            order = CounterTerranTie([RoachRavagerNydus()])
        elif self.build_name == "Macro":
            order = CounterTerranTie([MacroBuild()])
        else:
            order = CounterTerranTie([NydusRush()])

        await self.knowledge.ai.chat_send(self.build_name)
        return BuildOrder([order])

    def configure_managers(self) -> Optional[List[ManagerBase]]:
        return [OlScoutManager()]
