from sc2.ids.unit_typeid import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans import BuildOrder, SequentialList, Step, StepBuildGas
from sharpy.plans.acts import ActBuilding, Tech, Expand
from sharpy.plans.acts.zerg import AutoOverLord, MorphLair, ZergUnit
from sharpy.plans.require import All, Gas, UnitExists, Time
from sharpy.plans.tactics import PlanDistributeWorkers
from paul_plans.operations import BanelingDropBase, CustomInject


class FastBanelingDrop(BuildOrder):
    def __init__(self):
        drops = [
            Step(None, BanelingDropBase(), skip_until=Time(5 * 60)),
            Step(None, BanelingDropBase(), skip_until=Time(7 * 60)),
        ]
        # for i in range(5, 60):
        #     drops.append(Step(None, BanelingDropBase(), skip_until=Time(i * 60)))
        self.lings = ZergUnit(UnitTypeId.ZERGLING)
        self.banelings = ZergUnit(UnitTypeId.BANELING, to_count=0)
        opening = SequentialList(
            ZergUnit(UnitTypeId.DRONE, to_count=13, only_once=True),
            ZergUnit(UnitTypeId.OVERLORD, to_count=2, only_once=True),
            ZergUnit(UnitTypeId.DRONE, to_count=17, only_once=True),
            Expand(2, priority=True, consider_worker_production=False),
            ZergUnit(UnitTypeId.DRONE, to_count=18, only_once=True),
            StepBuildGas(1),
            ActBuilding(UnitTypeId.SPAWNINGPOOL),
            ZergUnit(UnitTypeId.DRONE, to_count=20, only_once=True),
            ZergUnit(UnitTypeId.OVERLORD, to_count=3, only_once=True),
            ZergUnit(UnitTypeId.ZERGLING, to_count=6),
            ZergUnit(UnitTypeId.QUEEN, to_count=1),
            Step(None, ZergUnit(UnitTypeId.QUEEN, to_count=2), skip_until=UnitExists(UnitTypeId.HATCHERY, 2)),
        )

        units = Step(
            None,
            BuildOrder(
                AutoOverLord(),
                drops,
                Step(None, ZergUnit(UnitTypeId.OVERLORDTRANSPORT, to_count=1), skip_until=UnitExists(UnitTypeId.LAIR)),
                self.banelings,
                self.lings,
            ),
            skip_until=UnitExists(UnitTypeId.QUEEN, count=2, include_pending=True),
        )

        research_tech = BuildOrder(
            Tech(UpgradeId.ZERGLINGMOVEMENTSPEED),
            Tech(UpgradeId.OVERLORDSPEED),
            Step(All(UnitExists(UnitTypeId.LAIR), Gas(150)), Tech(UpgradeId.CENTRIFICALHOOKS)),
        )

        tech_buildings = BuildOrder(
            Step(All(Gas(100)), MorphLair(),),
            Step(UnitExists(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.BANELINGNEST)),
        )
        super(FastBanelingDrop, self).__init__(
            [CustomInject(), opening, units, research_tech, tech_buildings, PlanDistributeWorkers()]
        )

    async def execute(self) -> bool:
        if self.get_count(UnitTypeId.BANELINGNEST, include_pending=False) and self.get_count(UnitTypeId.LAIR):
            self.banelings.to_count = self.get_count(UnitTypeId.ZERGLING) // 2
        return await super(FastBanelingDrop, self).execute()
