from sc2.ids.ability_id import AbilityId
from sc2.ids.unit_typeid import UnitTypeId
from sharpy.managers.build_detector import EnemyRushBuild
from sharpy.managers.combat2 import MoveType
from sharpy.managers.roles import UnitTask
from sharpy.plans import BuildOrder, SequentialList, Step, StepBuildGas
from sharpy.plans.acts import ActBuilding, Expand, DefensiveBuilding, DefensePosition
from sharpy.plans.acts.zerg import AutoOverLord, MorphLair, ZergUnit
from sharpy.plans.require import Time, UnitExists
from sharpy.plans.tactics import (
    PlanDistributeWorkers,
    PlanFinishEnemy,
    PlanWorkerOnlyDefense,
    PlanZoneAttack,
    PlanZoneDefense,
    PlanZoneGather,
)
from sharpy.plans.tactics.scouting import ScoutAroundMain, Scout, ScoutLocation
from sharpy.plans.tactics.zerg import InjectLarva

from paul_plans.operations import NydusTarget


class RoachRavagerNydus(BuildOrder):
    """
    Fast Roach/Ravager into a Nydus outside the enemy natural.
    """

    def __init__(self):
        opening = SequentialList(
            Step(
                UnitExists(UnitTypeId.SPAWNINGPOOL),
                DefensiveBuilding(UnitTypeId.SPINECRAWLER, DefensePosition.Entrance, to_count=3, to_base_index=1),
                skip_until=lambda k: k.build_detector.rush_build == EnemyRushBuild.RoachRush,
            ),
            ZergUnit(UnitTypeId.DRONE, to_count=13, only_once=True),
            ZergUnit(UnitTypeId.OVERLORD, to_count=2, only_once=True),
            ZergUnit(UnitTypeId.DRONE, to_count=17, only_once=True),
            ActBuilding(UnitTypeId.SPAWNINGPOOL),
            ZergUnit(UnitTypeId.DRONE, to_count=18, only_once=True),
            Expand(2, consider_worker_production=False, priority=True),
            StepBuildGas(1),
            ZergUnit(UnitTypeId.DRONE, to_count=18, only_once=True),
            ZergUnit(UnitTypeId.QUEEN, to_count=1),
            ZergUnit(UnitTypeId.ZERGLING, to_count=2),
            ActBuilding(UnitTypeId.ROACHWARREN),
            ZergUnit(UnitTypeId.DRONE, to_count=19),
            ZergUnit(UnitTypeId.OVERLORD, to_count=3),
            Step(UnitExists(UnitTypeId.HATCHERY, count=2), ZergUnit(UnitTypeId.QUEEN, to_count=2)),
            MorphLair(),
            Step(
                None,
                ActBuilding(UnitTypeId.NYDUSNETWORK),
                skip_until=UnitExists(UnitTypeId.LAIR, include_pending=False),
            ),
            StepBuildGas(3),
            ZergUnit(UnitTypeId.DRONE, to_count=25),
        )

        scouting = Scout({UnitTypeId.ZERGLING}, 1, ScoutAroundMain())

        post_order = BuildOrder(
            [NydusTarget(2), AutoOverLord(), ZergUnit(UnitTypeId.ROACH), ZergUnit(UnitTypeId.RAVAGER)]
        )

        self.attack = PlanZoneAttack(120)
        self.nydus_attack = PlanZoneAttack(1)
        attack_tactics = [
            PlanWorkerOnlyDefense(),
            PlanZoneGather(),
            PlanZoneDefense(),
            Step(
                None,
                self.attack,
                skip=UnitExists(UnitTypeId.NYDUSCANAL, include_pending=False, include_not_ready=False),
            ),
            Step(
                None,
                self.nydus_attack,
                skip_until=UnitExists(UnitTypeId.NYDUSCANAL, include_pending=False, include_not_ready=False),
            ),
            PlanFinishEnemy(),
        ]

        self.worker_distribution = PlanDistributeWorkers()

        super(RoachRavagerNydus, self).__init__(
            scouting, InjectLarva(), opening, post_order, attack_tactics, self.worker_distribution,
        )

    async def execute(self) -> bool:
        if self.cache.own(UnitTypeId.NYDUSCANAL).ready:
            # we're ready to attack
            self.combat.add_units(self.cache.own({UnitTypeId.ROACH, UnitTypeId.RAVAGER, UnitTypeId.ZERGLING}))
            self.combat.execute(self.attack._get_target(), MoveType.Assault)
            self.roles.set_tasks(
                UnitTask.Attacking, self.cache.own({UnitTypeId.ROACH, UnitTypeId.RAVAGER, UnitTypeId.ZERGLING})
            )

        # self.network_load_or_unload_all()
        self.canal_unload_all()
        self.catch_nearby_units()
        self.catch_units_exiting()
        return await super(RoachRavagerNydus, self).execute()

    # def network_load_or_unload_all(self):
    #     load = True
    #
    #     if self.get_count(UnitTypeId.NYDUSNETWORK, include_pending=False):
    #         for zone in self.zone_manager.expansion_zones:
    #             if zone.is_ours and zone.is_under_attack:
    #                 load = False
    #                 break
    #         if self.cache.own(UnitTypeId.NYDUSNETWORK).ready:
    #             network = self.cache.own(UnitTypeId.NYDUSNETWORK).ready.first
    #             if load:
    #                 idle_ling_queen = self.roles.get_types_from({UnitTypeId.ZERGLING, UnitTypeId.QUEEN}, UnitTask.Idle)
    #                 for u in idle_ling_queen.closer_than(30, network.position):
    #                     self.ai.do(network(AbilityId.LOAD_NYDUSNETWORK, u))
    #             else:
    #                 self.ai.do(network(AbilityId.UNLOADALL_NYDASNETWORK))

    def canal_unload_all(self):
        for canal in self.cache.own(UnitTypeId.NYDUSCANAL).ready:
            self.ai.do(canal(AbilityId.UNLOADALL_NYDUSWORM))

    def catch_units_exiting(self):
        for canal in self.cache.own(UnitTypeId.NYDUSCANAL).ready:
            close = self.cache.own(
                [UnitTypeId.ZERGLING, UnitTypeId.QUEEN, UnitTypeId.ROACH, UnitTypeId.RAVAGER]
            ).closer_than(2, canal.position)
            for u in close:
                self.combat.add_unit(u)

    def catch_nearby_units(self):
        if self.cache.own(UnitTypeId.NYDUSNETWORK) and self.cache.own(UnitTypeId.NYDUSCANAL):
            network = self.cache.own(UnitTypeId.NYDUSNETWORK).first
            for unit in self.cache.own([UnitTypeId.ZERGLING, UnitTypeId.ROACH, UnitTypeId.RAVAGER]):
                if unit.distance_to(network.position) < 5:
                    self.ai.do(network(AbilityId.LOAD_NYDUSNETWORK, unit))

    def manage_gas(self):
        if self.knowledge.build_detector.rush_build == EnemyRushBuild.RoachRush:
            self.worker_distribution.max_gas = 6
        else:
            self.worker_distribution.max_gas = None
