"""
Deprecated build for Ling/Bane/X.
"""

from paul_plans.operations import SpreadCreepV2
from sc2.ids.unit_typeid import UnitTypeId
from sc2.ids.upgrade_id import UpgradeId
from sharpy.plans import BuildOrder, SequentialList, Step, StepBuildGas
from sharpy.plans.acts import ActBuilding, Expand, Tech
from sharpy.plans.acts.zerg import AutoOverLord, MorphLair, ZergUnit
from sharpy.plans.require import UnitExists
from sharpy.plans.tactics import (
    PlanDistributeWorkers,
    PlanFinishEnemy,
    PlanWorkerOnlyDefense,
    PlanZoneAttack,
    PlanZoneGather,
    PlanZoneDefense,
)
from sharpy.plans.tactics.zerg import InjectLarva


class OldLingBane(BuildOrder):
    """
    Deprecated attempt at Ling/Bane/X composition.
    """
    MAX_WORKERS = 54

    def __init__(self):
        self.attack = PlanZoneAttack(50)
        self.expander = Expand(to_count=3, priority=True, consider_worker_production=False)
        self.gas_builder = StepBuildGas(to_count=2)
        self.ling = ZergUnit(UnitTypeId.ZERGLING)
        self.bane = ZergUnit(UnitTypeId.BANELING, to_count=0)
        self.queen = ZergUnit(UnitTypeId.QUEEN, to_count=3)
        self.drone = ZergUnit(UnitTypeId.DRONE, to_count=self.MAX_WORKERS)

        units = BuildOrder(
            [
                Step(UnitExists(UnitTypeId.SPAWNINGPOOL, include_pending=False), self.ling, skip=self.should_drone),
                Step(UnitExists(UnitTypeId.BANELINGNEST, include_pending=False), self.bane, skip=self.should_drone),
                Step(None, self.drone, skip_until=self.should_drone),
                Step(UnitExists(UnitTypeId.SPAWNINGPOOL), self.queen),
            ]
        )

        buildings = BuildOrder(
            SequentialList(ActBuilding(UnitTypeId.SPAWNINGPOOL), ActBuilding(UnitTypeId.BANELINGNEST)),
            self.expander,
            self.gas_builder,
            Step(UnitExists(UnitTypeId.SPAWNINGPOOL), MorphLair()),
        )

        upgrades = BuildOrder(Tech(UpgradeId.ZERGLINGMOVEMENTSPEED), Tech(UpgradeId.CENTRIFICALHOOKS))

        attack_tactics = [PlanWorkerOnlyDefense(), PlanZoneGather(), PlanZoneDefense(), self.attack, PlanFinishEnemy()]

        super().__init__(
            [
                AutoOverLord(),
                upgrades,
                buildings,
                units,
                attack_tactics,
                PlanDistributeWorkers(),
                InjectLarva(),
                SpreadCreepV2(),
            ]
        )

    async def execute(self) -> bool:
        """
        Set counts for macro/army.
        :return:
        """
        self.set_expander()
        self.set_gas()
        self.set_queen()
        self.set_army()
        # self.set_attack()
        return await super(OldLingBane, self).execute()

    def should_drone(self, _knowledge) -> bool:
        """
        Determine if we should drone.
        :param _knowledge:
        :return:
        """
        if self.knowledge.ai.supply_workers >= min(
            self.MAX_WORKERS, self.get_count(UnitTypeId.HATCHERY) * 16 + self.get_count(UnitTypeId.EXTRACTOR) * 3
        ):
            return False
        for zone in self.knowledge.expansion_zones:
            if zone.is_ours and zone.is_under_attack:
                return False
        return True

    def set_expander(self):
        """
        Set expansion count.
        :return:
        """
        mineral_count = 0
        for zone in self.knowledge.expansion_zones:
            if zone.is_ours:
                mineral_count += zone.mineral_fields.amount
        if mineral_count < self.get_count(UnitTypeId.HATCHERY) * 4:
            self.expander.to_count = self.get_count(UnitTypeId.HATCHERY) + 1

    def set_gas(self):
        """
        Set number of gases to take.
        :return:
        """
        gas_count = 0
        for geyser in self.cache.own(UnitTypeId.EXTRACTOR):
            gas_count += geyser.vespene_contents
        if gas_count < 400:
            self.gas_builder.to_count = self.get_count(UnitTypeId.EXTRACTOR) + 1

    def set_queen(self):
        """
        Set number of queens.
        :return:
        """
        self.queen.to_count = max(3, self.get_count(UnitTypeId.HATCHERY) + 1)

    def set_army(self):
        """
        Set baneling count to 1/4 army.
        :return:
        """
        if self.get_count(UnitTypeId.BANELINGNEST, include_pending=False):
            self.bane.to_count = self.cache.own(UnitTypeId.ZERGLING).amount // 3

    def set_attack(self):
        """
        Lower the attack trigger once Baneling speed is researched.
        :return:
        """
        if UpgradeId.CENTRIFICALHOOKS in self.knowledge.ai.state.upgrades:
            self.attack.start_attack_power = 20
