from typing import Dict, List, TYPE_CHECKING

from sc2.ids.unit_typeid import UnitTypeId
from sc2.player import Race
from sc2.position import Point2
from sharpy.managers.roles import UnitTask
from sharpy.managers import ManagerBase
from paul_plans.operations import OverlordSpots

if TYPE_CHECKING:
    from sharpy.knowledges import Knowledge


class OlScoutManager(ManagerBase):
    """
    Manage scouting, just Overlords for now.
    """
    def __init__(self):
        super(OlScoutManager, self).__init__()
        self.spots: List[Point2] = []
        self.ol_locations: Dict[int, Point2] = {}

    async def start(self, knowledge: "Knowledge"):
        await super(OlScoutManager, self).start(knowledge)
        ol_spots = OverlordSpots(self.knowledge.ai.game_info).spots
        ol_spots.sort(key=lambda s: s.distance_to(self.knowledge.enemy_expansion_zones[1].gather_point))
        self.spots.extend(ol_spots)

    async def update(self):
        if self.recall_overlords():
            for overlord in self.cache.own(UnitTypeId.OVERLORD):
                if self.knowledge.ai.game_info.pathing_grid[overlord.position.rounded]:
                    self.ai.do(overlord.move(self.knowledge.own_main_zone.behind_mineral_position_center))
        self.check_for_dead_overlords()
        self.assign_new_overlord()

    async def post_update(self):
        pass

    def check_for_dead_overlords(self):
        """
        Find dead overlords and replace them.
        :return:
        """
        dead_tags: List[int] = []
        for tag in self.ol_locations:
            if tag not in self.cache.own(UnitTypeId.OVERLORD).tags:
                dead_tags.append(tag)
        for tag in dead_tags:
            self.replace_overlord(tag)

    def assign_new_overlord(self):
        if len(self.ol_locations) >= len(self.spots):
            return
        available_overlords = self.roles.get_types_from({UnitTypeId.OVERLORD}, UnitTask.Reserved)
        for overlord in available_overlords:
            for spot in self.spots:
                if spot not in list(self.ol_locations.values()):
                    self.ol_locations[overlord.tag] = spot
                    self.roles.set_task(UnitTask.Scouting, overlord)
                    self.ai.do(overlord.move(spot))
                    break

    def replace_overlord(self, tag: int):
        if tag in self.ol_locations:
            location = self.ol_locations[tag]
            del self.ol_locations[tag]
            available_overlords = self.roles.get_types_from({UnitTypeId.OVERLORD}, UnitTask.Reserved)
            if available_overlords:
                overlord = available_overlords.closest_to(self.knowledge.expansion_zones[1].gather_point)
                if overlord:
                    self.roles.set_task(UnitTask.Scouting, overlord)
                    self.ol_locations[overlord.tag] = location
                    self.ai.do(overlord.move(location))

    def recall_overlords(self) -> bool:
        """
        Determine if we need to recall Overlords.
        :return:
        """
        if self.knowledge.enemy_units_manager.enemy_total_power.air_power >= 10:
            # enemy has air units (could be Banshees/Oracles, but w/e)
            return True
        if self.knowledge.build_detector.rush_build and self.knowledge.enemy_race != Race.Zerg:
            # Zerg rushes can't shoot air
            return True
        return False
