def calc_nydus_spot(self):
    # create KDTree for nearest neighbor searching composed only of points within the playable area
    x_start, y_start, x_dist, y_dist = self.knowledge.ai.game_info.playable_area
    standin = [(x, y) for x in range(x_start, x_start + x_dist + 1) for y in range(y_start, y_start + y_dist + 1)]
    tree = spatial.KDTree(standin)
    # find enemy main gas geysers and unique points within 9 of them (the typical sight radius of a gas building)
    enemy_gases = self.knowledge.ai.vespene_geyser.closer_than(12, self.knowledge.enemy_main_zone.center_location)
    gas_one = [standin[x] for x in tree.query_ball_point(enemy_gases[0].position, 9)]
    gas_two = [standin[y] for y in tree.query_ball_point(enemy_gases[1].position, 9)]
    close_to_gas = set(gas_one + gas_two)
    # get the height of the enemy main
    enemy_main_height = self.knowledge.ai.game_info.terrain_height.data_numpy[
        self.knowledge.enemy_main_zone.center_location.rounded
    ]
    # "find" the enemy main tiles based on tiles within distance that are the same height as the main
    enemy_main = [
        standin[z]
        for z in tree.query_ball_point(self.knowledge.enemy_main_zone.center_location, 45)
        if self.knowledge.ai.game_info.terrain_height.data_numpy[standin[z]] == enemy_main_height
           and self.knowledge.ai.game_info.pathing_grid.data_numpy[standin[z]] == 1
    ]
    # remove points that are too close to the gas geysers
    main_away_from_gas = list(set(enemy_main) - close_to_gas)
    # compute the distances of each remaining point to the enemy townhall
    distances = spatial.distance_matrix(main_away_from_gas, [self.knowledge.enemy_main_zone.center_location])
    # select the first point furthest from the enemy townhall
    nydus_raw_location = main_away_from_gas[np.where(distances == max(distances))[0][0]]
    # return the location as Point2
    return Point2(nydus_raw_location)