from typing import TYPE_CHECKING

from sc2.units import Units
from sc2.unit import Unit
from sc2.ids.ability_id import AbilityId
from sc2.ids.unit_typeid import UnitTypeId
from sc2.position import Point2
from sharpy.general.zone import Zone
from sharpy.plans.acts import ActBase

if TYPE_CHECKING:
    from sharpy.knowledges import Knowledge


class NydusReinforcements(ActBase):
    """
    Place Nydus near army for faster reinforcements.

    Only works if a Nydus Network already exists.
    """

    async def start(self, knowledge: "Knowledge"):
        await super(NydusReinforcements, self).start(knowledge)

    async def execute(self) -> bool:
        if not self.get_count(UnitTypeId.NYDUSNETWORK, include_pending=False):
            # we don't have a Nydus and this Act won't build one.
            return True

        if not self.get_count(UnitTypeId.NYDUSCANAL):
            # no Nydus in play, let's get one on the map
            self.place_start_canal()
            return True

    def place_start_canal(self):
        """
        Place an initial canal, even if we're not planning on attacking yet
        :return:
        """
        if not self.ai.can_afford(UnitTypeId.NYDUSCANAL):
            # can't afford a canal, move along
            return

        units: Units = self.cache.own_in_range(self.knowledge.enemy_expansion_zones[2], 50)
        if not units:
            # we don't have units close enough
            return

        spotter_unit: Unit = units.random
        spotter_pos: Point2 = spotter_unit.position
        for i in range(spotter_unit.sight_range - 1):
            # slightly under sight range, just in case
            loc = spotter_pos.towards_with_random_angle(self.knowledge.enemy_expansion_zones[2], i)
            if await self.ai.can_place(UnitTypeId.NYDUSCANAL, loc):
                network = self.cache.own(UnitTypeId.NYDUSNETWORK)
                self.ai.do(network(AbilityId.BUILD_NYDUSWORM, loc))
                return

        return

    def place_reinforcement_canal(self):
        """
        Place canal behind own troops.
        :return:
        """
        group = self.combat.closest_group()